# -*- coding: utf-8 -*-
from __future__ import print_function

from homura import download
import requests
import json

from datetime import datetime, date, timedelta
from os.path import join, exists, getsize


def format_date(in_date):
    """Format date or datetime input to YYYY-MM-DDThh:mm:ssZ"""

    if type(in_date) == datetime or type(in_date) == date:
        return in_date.strftime('%Y-%m-%dT%H:%M:%SZ')
    else:
        try:
            return datetime.strptime(in_date, '%Y%m%d').strftime('%Y-%m-%dT%H:%M:%SZ')
        except ValueError:
            return in_date


class SentinelAPI(object):
    """Class to connect to Sentinel-1 Scientific Data Hub, search and download
    imagery.
    """
    def __init__(self, user, password):
        self.session = requests.Session()
        self.session.auth = (user, password)

    def query(self, area, initial_date=None, end_date=datetime.now(), **keywords):
        """Call the Scihub"""
        self.format_url(area, initial_date, end_date, **keywords)
        self.content = self.session.get(self.url)
        if self.content.status_code != 200:
            print(('Query returned %s error.' % self.content.status_code))

    def format_url(self, area, initial_date=None, end_date=datetime.now(), **keywords):
        """Create the URL to access the SciHub API."""
        if initial_date is None:
            initial_date = end_date - timedelta(hours=24)

        ingestion_date = '(ingestionDate:[%s TO %s])' % (
            format_date(initial_date),
            format_date(end_date)
        )
        query_area = ' AND (footprint:"Intersects(POLYGON((%s)))")' % area

        filters = ''
        for kw in sorted(keywords.keys()):
            filters += ' AND (%s:%s)' % (kw, keywords[kw])

        self.url = 'https://scihub.esa.int/dhus/search?format=json&q=%s%s%s' \
            % (ingestion_date, query_area, filters)

    def get_products(self):
        """Return the ids of the products found in the Query."""
        try:
            self.products = self.content.json()['feed']['entry']
            return self.products
        except KeyError:
            print('No products found in this query.')
            return []

    def get_product_info(self, id):
        """Access SciHub API to get info about a Product. Returns a dict with
        the id, title, size and download url of the Product.
        """

        product = self.session.get(
            "https://scihub.esa.int/dhus/odata/v1/Products('%s')/?$format=json" % id
            )
        product_json = product.json()
        keys = ['id', 'title', 'size', 'url']
        values = [
            product_json['d']['Id'],
            product_json['d']['Name'],
            int(product_json['d']['ContentLength']),
            "https://scihub.esa.int/dhus/odata/v1/Products('%s')/$value" % id
            ]
        return dict(zip(keys, values))

    def download(self, id, path='.'):
        """Download a product using homura's download function. If you don't
        pass the title of the product, it will use the id as filename.
        """
        product = self.get_product_info(id)
        path = join(path, product['title'] + '.zip')

        print('Downloading %s to %s' % (id, path))

        # Check if the file exists and if it is complete
        if exists(path):
            if getsize(path) == product['size']:
                print('%s was already downloaded.' % path)
                return path

        download(product['url'], path=path, session=self.session)
        return path

    def download_all(self, path='.'):
        for product in self.get_products():
            self.download(product['id'], path)


def get_coordinates(geojson_file, feature_number=0):
    geojson = json.loads(open(geojson_file, 'r').read())
    coordinates = geojson['features'][feature_number]['geometry']['coordinates'][0]
    coordinates = ['%s %s' % tuple(coord) for coord in coordinates]
    return ','.join(coordinates)


import click

from sentinelsat.sentinel import SentinelAPI, get_coordinates


@click.group()
def cli():
    pass


@cli.command()
@click.argument('user', type=str, metavar='<user>')
@click.argument('password', type=str, metavar='<password>')
@click.argument('geojson', type=click.Path(exists=True), metavar='<geojson>')
@click.option('--start', '-s', type=str, default='NOW-1DAY',
    help='Start date of the query in the format YYYY-MM-DD.')
@click.option('--end', '-e', type=str, default='NOW',
    help='End date of the query in the format YYYY-MM-DD.')
@click.option('--download', '-d', is_flag=True,
    help='Download all the results of the query.')
@click.option('--path', '-p', type=click.Path(exists=True), default='.',
    help='Set the path where the files will be saved.')
def search(user, password, geojson, start, end, download, path):
    """Search for Sentinel-1 products and, optionally, download all the results.
    Beyond your scihub user and password, you must pass a geojson file
    containing the polygon of the area where you want to search for. If you
    don't especify the start and end dates, it will search in the last 24 hours.
    """
    api = SentinelAPI(user, password)
    api.query(get_coordinates(geojson), start, end)
    if download is True:
        api.download_all(path)
    else:
        for product in api.get_products():
            print('Product %s - %s' % (product['id'], product['summary']))


@cli.command()
@click.argument('user', type=str, metavar='<user>')
@click.argument('password', type=str, metavar='<password>')
@click.argument('productid', type=str, metavar='<productid>')
@click.option('--path', '-p', type=click.Path(exists=True), default='.',
    help='Set the path where the files will be saved.')
def download(user, password, productid, path):
    """Download a Sentinel-1 Product. Just needs you scihub user and password
    and the id of the product you want to download.
    """
    api = SentinelAPI(user, password)
    api.download(productid, path)


from codecs import open as codecs_open
from setuptools import setup, find_packages


# Get the long description from the relevant file
with codecs_open('README.rst', encoding='utf-8') as f:
    long_description = f.read()


setup(name='sentinelsat',
      version='0.1.1',
      description="Utility to search and download Sentinel-1 Imagery",
      long_description=long_description,
      classifiers=[
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.4',
          'Topic :: Scientific/Engineering :: GIS',
          'Topic :: Utilities',
      ],
      keywords='',
      author="Wille Marcel",
      author_email='wille@wille.blog.br',
      url='https://github.com/ibamacsr/sentinelsat',
      license='GPLv3+',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'requests',
          'click',
          'homura'
      ],
      extras_require={
          'test': ['pytest'],
      },
      entry_points="""
      [console_scripts]
      sentinel=sentinelsat.scripts.cli:cli
      """
      )
